import numpy as np
from src.plot import plot_xy, plot_color2d, plot_3d
import time
from src.TiseSolverFgh2D import TiseSolverFgh2D
import os
import sys
from pathlib import Path
import matplotlib.colors as mcolors


# BLAS ライブラリ用のスレッド数設定
os.environ["MKL_NUM_THREADS"] = "4"  # Intel MKL を使用する場合
os.environ["OMP_NUM_THREADS"] = "4"  # OpenMP を使用する場合
os.environ["OPENBLAS_NUM_THREADS"] = "4"  # OpenBLAS を使用する場合


if __name__ == "__main__":
    # - Input parameters
    # -- Position
    xa = -6.0  # Position x = [xa, xb]
    xb = 6.0
    nx = 2**5  # Number of x points
    # -- Position
    ya = -6.0  # Position x = [xa, xb]
    yb = 6.0
    ny = 2**5  # Number of y points

    # -- How many eigenvalues and eigenvectors you want from the smallest eigenvalue? (num_eig < nx*ny)
    num_eig = 21

    # -- Eigenfunction abscissa
    x = np.linspace(xa, xb, 6 * nx, endpoint=False, dtype=np.float64)
    y = np.linspace(ya, yb, 6 * ny, endpoint=False, dtype=np.float64)
    x_mesh, y_mesh = np.meshgrid(x, y, indexing="ij")  # X, Y has shapes (len(x), len(y))

    # -- Output folder
    output_folder_path = "./out/"

    # -- Preparetion
    folder_path = Path(output_folder_path)  # Make folder
    folder_path.mkdir(exist_ok=True, parents=True)

    # =====================
    #         TISE
    # =====================

    # -- Solve TISE using Fourier-Grid Hamiltonian method
    solver = TiseSolverFgh2D(xa, xb, nx, ya, yb, ny, num_eig)
    # x, y, Vxy, x_mesh, y_mesh = solver.abscissa_potential()
    start_time = time.time()  # Record the start time
    eigenvalues = solver.solve()
    eigenfunctions = solver.eigenfunctions_scaling(x, y)
    end_time = time.time()  # Record the end time
    print(f"  Elapsed time TISE: {end_time - start_time:.6f} seconds")
    for i in range(15):
        print(i, eigenvalues[i].real)

    # -- Check --
    # Direct integral by rectangular or trapezoidal rules.
    # dx = x[1] - x[0]
    # dy = y[1] - y[0]
    # for n in range(eigenfunctions.shape[0]):
    #     s_rect = dx * dy * np.sum(np.abs(eigenfunctions[n, :, :]) ** 2)
    #     s_trap = np.trapezoid(np.trapezoid(np.abs(eigenfunctions[n, :, :]) ** 2, dx=dy, axis=1), dx=dx, axis=0)
    #     print(n, s_rect, s_trap)

    # -- Output TISE results
    for n in range(15):
        x_dict = y_dict = z_dict = {}
        x_dict = {"plot": x_mesh}
        y_dict = {"plot": y_mesh}
        z_dict = {"plot": eigenfunctions[n, :, :].real}

        output_image_name = os.path.join(output_folder_path, f"eigenfunctions_n{n}.png")

        # color map
        parula_colors = ["#352a87", "#2053d4", "#0d75dc", "#0c93d2", "#07a9c2", "#38b99e", "#7cbf7b", "#b7bd64", "#f1b94a", "#fad32a", "#f9fb0e"]
        parula_cmap = mcolors.LinearSegmentedColormap.from_list("custom_cmap", parula_colors, N=256)
        jet_colors = ["#0fffee", "#0090ff", "#000fff", "#000090", "#ffffff", "#7f0000", "#ee0000", "#ff7000", "#ffee00"]
        jet_cmap = mcolors.LinearSegmentedColormap.from_list("custom_cmap", jet_colors, N=256)

        plot_color2d(
            x_mesh,
            y_mesh,
            eigenfunctions[n, :, :].real,
            output_image_name=output_image_name,
            color_range=(-0.6, 0.6),
            show_gui=False,
            color_map_name=jet_cmap,
            aspect_ratio="equal",
        )

    sys.exit()
